/*
 * libdigidocpp
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "MSX509CertStore.h"

#include "log.h"

#include <Windows.h>

using namespace digidoc;

/**
 * Loads all certificates from system store and adds these to the certificate store.
 *
 * @throws IOException exception is throws if failed to open certstore
 */
MSX509CertStore::MSX509CertStore()
{
    loadCerts("ROOT");
    loadCerts("CA");
    INFO("Loaded %d certificates into certificate store.", certs().size());
}

/**
 * Load all certificates found in CertStore and adds these to the cert store.
 *
 * @param provider name.
 * @throws IOException exception is throws if failed open CertStore.
 */
void MSX509CertStore::loadCerts(const std::string &provider)
{
    HCERTSTORE s = CertOpenStore(CERT_STORE_PROV_SYSTEM_A,
        X509_ASN_ENCODING, 0, CERT_SYSTEM_STORE_CURRENT_USER, provider.c_str() );
    if(!s)
        THROW("Failed to open CertStore with provider %s, can not load cert store.", provider.c_str());

    PCCERT_CONTEXT pc = nullptr;
    while((pc = CertEnumCertificatesInStore(s, pc)))
        addCert(X509Cert(std::vector<unsigned char>(pc->pbCertEncoded, pc->pbCertEncoded+pc->cbCertEncoded)));
    CertCloseStore(s, 0);
}
